@@ -191,15 +191,12 @@ Metaclass
 =========
 
 .. autoclass:: Meta()
-
-
-"""
+"""  # NOQA: E501
 
 import abc
 import asyncio
 import collections
 import contextlib
-import copy
 import logging
 import warnings
 import weakref
@@ -210,7 +207,7 @@ import aioxmpp.stream
 
 def automake_magic_attr(obj):
     obj._aioxmpp_service_handlers = getattr(
-        obj, "_aioxmpp_service_handlers", set()
+        obj, "_aioxmpp_service_handlers", {}
     )
     return obj._aioxmpp_service_handlers
 
@@ -329,60 +326,6 @@ class Descriptor(metaclass=abc.ABCMeta):
         """
 
 
-class DependencyGraphNode:
-    def __init__(self):
-        self.class_ = None
-
-    def __repr__(self):
-        return "DependencyGraphNode({!r})".format(self.class_)
-
-
-class DependencyGraph:
-
-    def __init__(self, edges=None, nodes=None):
-        if edges is None:
-            edges = []
-        if nodes is None:
-            nodes = []
-        self._edges = set(edges)
-        self._nodes = set(nodes)
-
-    def __deepcopy__(self, memo):
-        return DependencyGraph(self._edges, self._nodes)
-
-    def add_node(self, node):
-        self._nodes.add(node)
-
-    def add_edge(self, from_, to):
-        self._edges.add((from_, to))
-
-    def toposort(self):
-        edges = collections.defaultdict(lambda: set())
-        for from_, to in self._edges:
-            edges[from_].add(to)
-
-        sorted_ = []
-        marked = set()
-        done = set()
-
-        def visit(node):
-            if node in marked:
-                raise ValueError("dependency loop in service definitions")
-            if node in done:
-                return
-            done.add(node)
-            marked.add(node)
-            for dep in edges[node]:
-                visit(dep)
-            marked.remove(node)
-            sorted_.append(node)
-
-        for node in self._nodes:
-            visit(node)
-
-        return sorted_
-
-
 class Meta(abc.ABCMeta):
     """
     The metaclass for services. The :class:`Service` class uses it and in
@@ -459,22 +402,25 @@ class Meta(abc.ABCMeta):
 
        .. versionadded:: 0.9
 
-    .. attribute:: _DEPGRAPH_NODE
-
-       An internal token used for topological ordering. Consider this
-       name reserved by the metaclass.
-
-       It is an error to manually define :attr:`_DEPGRAPH_NODE` in a class
-       definition, doing so will raise a :class:`TypeError`.
-
-       .. versionadded:: 0.9
-
     .. versionchanged:: 0.9
 
        The :attr:`ORDER_AFTER` and :attr:`ORDER_BEFORE` attribtes do not
        change after class creation. In earlier versions they contained
        the transitive completion of the dependency relation.
 
+    The following attribute was generated in earlier version of
+    aioxmpp:
+
+    .. attribute:: _DEPGRAPH_NODE
+
+       For compatibility with earlier versions, a warning is issued
+       when :attr:`_DEPGRAPH_NODE` is defined in a service class
+       definition.
+
+       This behaviour will be removed in aioxmpp 1.0.
+
+       .. deprecated:: 0.11
+
     Dependency relationships must not have cycles; a cycle results in a
     :class:`ValueError` when the class causing the cycle is declared.
 
@@ -502,12 +448,8 @@ class Meta(abc.ABCMeta):
     ``Baz`` and ``Fourth`` will be instanciated before ``Bar`` and ``Bar`` will
     be instanciated before ``Foo``. There is no dependency relationship between
     ``Baz`` and ``Fourth``.
-
     """
 
-    __dependency_graph = DependencyGraph()
-    __service_order = {}
-
     def __new__(mcls, name, bases, namespace, inherit_dependencies=True):
         if "SERVICE_BEFORE" in namespace or "SERVICE_AFTER" in namespace:
             if "ORDER_BEFORE" in namespace or "ORDER_AFTER" in namespace:
@@ -532,9 +474,17 @@ class Meta(abc.ABCMeta):
             )
 
         if "_DEPGRAPH_NODE" in namespace:
+            warnings.warn(
+                "_DEPGRAPH_NODE should not be defined manually. "
+                "In version before 0.11 it was supplied automatically by "
+                "the metaclass and defining it raised TypeError."
+            )
+
+        if any(isinstance(mcls, base)
+               for base in bases) and "service_order_index" in namespace:
             raise TypeError(
-                "_DEPGRAPH_NODE must not be defined manually. "
-                "it is supplied automatically by the metaclass."
+                "service_order_index must not be defined manually. "
+                "It is supplied automatically by the metaclass."
             )
 
         for base in bases:
@@ -555,19 +505,12 @@ class Meta(abc.ABCMeta):
             namespace.get("ORDER_AFTER", ()))
         namespace["PATCHED_ORDER_AFTER"] = namespace["ORDER_AFTER"]
 
-        new_deps = copy.deepcopy(mcls.__dependency_graph)
-
-        depgraph_node = namespace["_DEPGRAPH_NODE"] = DependencyGraphNode()
-        new_deps.add_node(depgraph_node)
-        for cls in namespace["ORDER_AFTER"]:
-            new_deps.add_edge(depgraph_node, cls._DEPGRAPH_NODE)
-        for cls in namespace["ORDER_BEFORE"]:
-            new_deps.add_edge(cls._DEPGRAPH_NODE, depgraph_node)
-        sorted_ = new_deps.toposort()
-        mcls.__dependency_graph = new_deps
-        mcls.__service_order = dict(
-            (node, i) for i, node in enumerate(sorted_)
-        )
+        if namespace["ORDER_BEFORE"] and namespace["ORDER_AFTER"]:
+            visited = set()
+            for item in namespace["PATCHED_ORDER_AFTER"]:
+                if item.orders_after_any(namespace["ORDER_BEFORE"],
+                                         visited=visited):
+                    raise ValueError("dependency loop in service definitions")
 
         SERVICE_HANDLERS = []
         existing_handlers = set()
@@ -587,7 +530,7 @@ class Meta(abc.ABCMeta):
                     key = next(iter(conflicting))
                     obj = next(iter(
                         obj
-                        for obj_key, obj in SERVICE_HANDLERS
+                        for obj_key, obj, _ in SERVICE_HANDLERS
                         if obj_key == key
                     ))
 
@@ -602,7 +545,7 @@ class Meta(abc.ABCMeta):
 
                 existing_handlers |= unique_handlers
 
-                for spec in new_handlers:
+                for spec, kwargs in new_handlers.items():
                     missing = spec.require_deps - namespace["ORDER_AFTER"]
                     if missing:
                         raise TypeError(
@@ -613,7 +556,7 @@ class Meta(abc.ABCMeta):
                         )
 
                     SERVICE_HANDLERS.append(
-                        (spec.key, attr_value)
+                        (spec.key, attr_value, kwargs)
                     )
 
             elif isinstance(attr_value, Descriptor):
@@ -637,7 +580,6 @@ class Meta(abc.ABCMeta):
         super().__init__(name, bases, namespace)
         for cls in self.ORDER_BEFORE:
             cls.PATCHED_ORDER_AFTER |= frozenset([self])
-        self._DEPGRAPH_NODE.class_ = self
 
     def __prepare__(*args, **kwargs):
         return collections.OrderedDict()
@@ -650,13 +592,58 @@ class Meta(abc.ABCMeta):
     def SERVICE_AFTER(self):
         return self.ORDER_AFTER
 
-    def __lt__(self, other):
-        return (self.__service_order[self._DEPGRAPH_NODE] <
-                self.__service_order[other._DEPGRAPH_NODE])
+    def orders_after(self, other, *, visited=None):
+        """
+        Return whether `self` depends on `other` and will be instanciated
+        later.
 
-    def __le__(self, other):
-        return (self.__service_order[self._DEPGRAPH_NODE] <=
-                self.__service_order[other._DEPGRAPH_NODE])
+        :param other: Another service.
+        :type other: :class:`aioxmpp.service.Service`
+
+        .. versionadded:: 0.11
+        """
+        return self.orders_after_any(frozenset([other]), visited=visited)
+
+    def orders_after_any(self, other, *, visited=None):
+        """
+        Return whether `self` orders after any of the services in the set
+        `other`.
+
+        :param other: Another service.
+        :type other: A :class:`set` of
+          :class:`aioxmpp.service.Service` instances
+
+        .. versionadded:: 0.11
+        """
+        if not other:
+            return False
+        if visited is None:
+            visited = set()
+        elif self in visited:
+            return False
+        visited.add(self)
+        for item in self.PATCHED_ORDER_AFTER:
+            if item in visited:
+                continue
+            if item in other:
+                return True
+            if item.orders_after_any(other, visited=visited):
+                return True
+        return False
+
+    def independent_from(self, other):
+        """
+        Return whether the services are independent (neither depends on
+        the other).
+
+        :param other: Another service.
+        :type other: :class:`aioxmpp.service.Service`
+
+        .. versionadded:: 0.11
+        """
+        if self is other:
+            return False
+        return not self.orders_after(other) and not other.orders_after(self)
 
 
 class Service(metaclass=Meta):
@@ -686,7 +673,7 @@ class Service(metaclass=Meta):
 
     To stay forward compatible, accept arbitrary keyword arguments and pass
     them down to :class:`Service`. As it is not possible to directly pass
-    arguments to :class:`Service`\ s on construction (due to the way
+    arguments to :class:`Service`\\ s on construction (due to the way
     :meth:`aioxmpp.Client.summon` works), there is no need for you
     to introduce custom arguments, and thus there should be no conflicts.
 
@@ -700,12 +687,15 @@ class Service(metaclass=Meta):
 
     .. autoattribute:: dependencies
 
+    .. autoattribute:: service_order_index
+
     .. automethod:: derive_logger
 
     .. automethod:: shutdown
     """
 
-    def __init__(self, client, *, logger_base=None, dependencies={}):
+    def __init__(self, client, *, logger_base=None, dependencies={},
+                 service_order_index=0):
         if logger_base is None:
             self.logger = logging.getLogger(".".join([
                 type(self).__module__, type(self).__qualname__
@@ -717,21 +707,35 @@ class Service(metaclass=Meta):
         self.__context = contextlib.ExitStack()
         self.__client = client
         self.__dependencies = dependencies
+        self.__service_order_index = service_order_index
 
         for item in self.SERVICE_HANDLERS:
             if isinstance(item, Descriptor):
                 item.add_to_stack(self, self.__context)
             else:
-                (handler_cm, additional_args), obj = item
+                (handler_cm, additional_args), obj, kwargs = item
                 self.__context.enter_context(
                     handler_cm(
                         self,
                         self.__client.stream,
                         obj.__get__(self, type(self)),
-                        *additional_args
+                        *additional_args,
+                        **kwargs
                     )
                 )
 
+    @property
+    def service_order_index(self):
+        """
+        Return the index of this service in the toposort of summoned
+        services. This is primarily used to order filter chain
+        registrations consistently with the dependency relationship of
+        the services.
+
+        .. versionadded:: 0.11
+        """
+        return self.__service_order_index
+
     def derive_logger(self, logger):
         """
         Return a child of `logger` specific for this instance. This is called
@@ -774,8 +778,7 @@ class Service(metaclass=Meta):
         """
         return self.__dependencies
 
-    @asyncio.coroutine
-    def _shutdown(self):
+    async def _shutdown(self):
         """
         Actual implementation of the shut down process.
 
@@ -784,8 +787,7 @@ class Service(metaclass=Meta):
         instead of :meth:`shutdown`.
         """
 
-    @asyncio.coroutine
-    def shutdown(self):
+    async def shutdown(self):
         """
         Close the service and wait for it to completely shut down.
 
@@ -800,7 +802,7 @@ class Service(metaclass=Meta):
            override the :meth:`_shutdown` method.
 
         """
-        yield from self._shutdown()
+        await self._shutdown()
         self.__context.close()
         self.__client = None
 
@@ -854,29 +856,62 @@ class HandlerSpec(collections.namedtuple(
     If at class definition time any of the dependent classes in `require_deps`
     are not declared using the order attributes (see :class:`Meta`), a
     :class:`TypeError` is raised.
+
+    There is a property to extract the function directly:
+
+    .. autoattribute:: func
     """
 
     def __new__(cls, key, is_unique=True, require_deps=()):
         return super().__new__(cls, is_unique, key, frozenset(require_deps))
 
+    @property
+    def func(self):
+        """
+        The factory of the context manager for this handler.
 
-def add_handler_spec(f, handler_spec):
+        .. versionadded:: 0.11
+        """
+        return self.key[0]
+
+
+def add_handler_spec(f, handler_spec, *, kwargs=None):
     """
     Attach a handler specification (see :class:`HandlerSpec`) to a function.
 
     :param f: Function to attach the handler specification to.
     :param handler_spec: Handler specification to attach to the function.
     :type handler_spec: :class:`HandlerSpec`
+    :param kwargs: additional keyword arguments passed to the function
+       carried in the handler spec.
+    :type kwargs: :class:`dict`
+
+    :raises ValueError: if the handler was registered with
+       different `kwargs` before
 
     This uses a private attribute, whose exact name is an implementation
-    detail. The `handler_spec` is stored in a :class:`set` bound to the
+    detail. The `handler_spec` is stored in a :class:`dict` bound to the
     attribute.
+
+    .. versionadded:: 0.11
+
+       The `kwargs` argument. If two handlers with the same spec, but
+       different arguments are registered for one function, an error
+       will be raised. So you should always include all possible
+       arguments, this is the responsibility of the calling decorator.
     """
-    automake_magic_attr(f).add(handler_spec)
+    handler_dict = automake_magic_attr(f)
+    if kwargs is None:
+        kwargs = {}
+    if kwargs != handler_dict.setdefault(handler_spec, kwargs):
+        raise ValueError(
+            "The additional keyword arguments to the handler are incompatible")
 
 
-def _apply_iq_handler(instance, stream, func, type_, payload_cls):
-    return aioxmpp.stream.iq_handler(stream, type_, payload_cls, func)
+def _apply_iq_handler(instance, stream, func, type_, payload_cls, *,
+                      with_send_reply=False):
+    return aioxmpp.stream.iq_handler(stream, type_, payload_cls, func,
+                                     with_send_reply=with_send_reply)
 
 
 def _apply_presence_handler(instance, stream, func, type_, from_):
@@ -887,7 +922,7 @@ def _apply_inbound_message_filter(instance, stream, func):
     return aioxmpp.stream.stanza_filter(
         stream.service_inbound_message_filter,
         func,
-        type(instance),
+        instance.service_order_index,
     )
 
 
@@ -895,7 +930,7 @@ def _apply_inbound_presence_filter(instance, stream, func):
     return aioxmpp.stream.stanza_filter(
         stream.service_inbound_presence_filter,
         func,
-        type(instance),
+        instance.service_order_index,
     )
 
 
@@ -903,7 +938,7 @@ def _apply_outbound_message_filter(instance, stream, func):
     return aioxmpp.stream.stanza_filter(
         stream.service_outbound_message_filter,
         func,
-        type(instance),
+        instance.service_order_index,
     )
 
 
@@ -911,7 +946,7 @@ def _apply_outbound_presence_filter(instance, stream, func):
     return aioxmpp.stream.stanza_filter(
         stream.service_outbound_presence_filter,
         func,
-        type(instance),
+        instance.service_order_index,
     )
 
 
@@ -942,7 +977,7 @@ def _apply_connect_depfilter(instance, stream, func, dependency, filter_name):
     else:
         dependency = instance.dependencies[dependency]
     filter_ = getattr(dependency, filter_name)
-    return filter_.context_register(func, type(instance))
+    return filter_.context_register(func, instance.service_order_index)
 
 
 def _apply_connect_attrsignal(instance, stream, func, descriptor, signal_name,
@@ -961,7 +996,7 @@ def _apply_connect_attrsignal(instance, stream, func, descriptor, signal_name,
         return signal.context_connect(func, mode)
 
 
-def iq_handler(type_, payload_cls):
+def iq_handler(type_, payload_cls, *, with_send_reply=False):
     """
     Register the decorated function or coroutine function as IQ request
     handler.
@@ -970,6 +1005,10 @@ def iq_handler(type_, payload_cls):
     :type type_: :class:`~.IQType`
     :param payload_cls: Payload XSO class to listen for
     :type payload_cls: :class:`~.XSO` subclass
+    :param with_send_reply: Whether to pass a function to send a reply
+       to the decorated callable as second argument.
+    :type with_send_reply: :class:`bool`
+
     :raises ValueError: if `payload_cls` is not a registered IQ payload
 
     If the decorated function is not a coroutine function, it must return an
@@ -977,17 +1016,22 @@ def iq_handler(type_, payload_cls):
 
     .. seealso::
 
-        :meth:`~.StanzaStream.register_iq_request_handler`
-            for more details on the `type_` and `payload_cls` arguments, as well
-            as behaviour expected from the decorated function.
+        :meth:`~.StanzaStream.register_iq_request_handler` for more
+            details on the `type_`, `payload_cls` and
+            `with_send_reply` arguments, as well as behaviour expected
+            from the decorated function.
+
         :meth:`aioxmpp.IQ.as_payload_class`
             for a way to register a XSO as IQ payload
 
+    .. versionadded:: 0.11
+
+       The `with_send_reply` argument.
+
     .. versionchanged:: 0.10
 
         The decorator now checks if `payload_cls` is a valid, registered IQ
         payload and raises :class:`ValueError` if not.
-
     """
 
     if (not hasattr(payload_cls, "TAG") or
@@ -1006,8 +1050,9 @@ def iq_handler(type_, payload_cls):
             f,
             HandlerSpec(
                 (_apply_iq_handler, (type_, payload_cls)),
-                require_deps=()
-            )
+                require_deps=(),
+            ),
+            kwargs=dict(with_send_reply=with_send_reply),
         )
         return f
     return decorator
@@ -1358,10 +1403,10 @@ def depfilter(class_, filter_name):
     return decorator
 
 
-def is_iq_handler(type_, payload_cls, coro):
+def is_iq_handler(type_, payload_cls, coro, *, with_send_reply=False):
     """
     Return true if `coro` has been decorated with :func:`iq_handler` for the
-    given `type_` and `payload_cls`.
+    given `type_` and `payload_cls` and the specified keyword arguments.
     """
 
     try:
@@ -1369,9 +1414,14 @@ def is_iq_handler(type_, payload_cls, coro):
     except AttributeError:
         return False
 
-    return HandlerSpec(
+    hs = HandlerSpec(
         (_apply_iq_handler, (type_, payload_cls)),
-    ) in handlers
+    )
+
+    try:
+        return handlers[hs] == dict(with_send_reply=with_send_reply)
+    except KeyError:
+        return False
 
 
 def is_message_handler(type_, from_, cb):
@@ -1404,9 +1454,11 @@ def is_inbound_message_filter(cb):
     except AttributeError:
         return False
 
-    return HandlerSpec(
+    hs = HandlerSpec(
         (_apply_inbound_message_filter, ())
-    ) in handlers
+    )
+
+    return hs in handlers
 
 
 def is_inbound_presence_filter(cb):
@@ -1420,9 +1472,11 @@ def is_inbound_presence_filter(cb):
     except AttributeError:
         return False
 
-    return HandlerSpec(
+    hs = HandlerSpec(
         (_apply_inbound_presence_filter, ())
-    ) in handlers
+    )
+
+    return hs in handlers
 
 
 def is_outbound_message_filter(cb):
@@ -1436,9 +1490,11 @@ def is_outbound_message_filter(cb):
     except AttributeError:
         return False
 
-    return HandlerSpec(
+    hs = HandlerSpec(
         (_apply_outbound_message_filter, ())
-    ) in handlers
+    )
+
+    return hs in handlers
 
 
 def is_outbound_presence_filter(cb):
@@ -1452,9 +1508,11 @@ def is_outbound_presence_filter(cb):
     except AttributeError:
         return False
 
-    return HandlerSpec(
+    hs = HandlerSpec(
         (_apply_outbound_presence_filter, ())
-    ) in handlers
+    )
+
+    return hs in handlers
 
 
 def is_depsignal_handler(class_, signal_name, cb, *, defer=False):
